@@ -4,32 +4,25 @@ from typing import (
     TYPE_CHECKING,
     Iterable,
     List,
-    Tuple,
     Set,
     TextIO,
     Any,
     Dict,
     Optional,
     Callable,
-    cast,
 )
 import sys
 from enum import IntEnum
-from itertools import permutations
 from ezdxf.lldxf import const, validator
 from ezdxf.entities import factory, DXFEntity
-from ezdxf.math import NULLVEC, Vec3, Matrix44
-from ezdxf.sections.table import table_key
+from ezdxf.math import NULLVEC
 
 if TYPE_CHECKING:
     from ezdxf.eztypes import (
-        DXFEntity,
         Drawing,
         DXFGraphic,
         BlocksSection,
         EntityDB,
-        Dimension,
-        GenericLayoutType,
     )
 
 __all__ = ["Auditor", "AuditError", "audit", "BlockCycleDetector"]
@@ -47,6 +40,9 @@ class AuditError(IntEnum):
     FOUND_TAG_OUTSIDE_SECTION = 8
     REMOVED_UNSUPPORTED_SECTION = 9
     REMOVED_UNSUPPORTED_TABLE = 10
+    REMOVED_INVALID_GRAPHIC_ENTITY = 11
+    REMOVED_INVALID_DXF_OBJECT = 12
+    REMOVED_STANDALONE_ATTRIB_ENTITY = 13
 
     UNDEFINED_LINETYPE = 100
     UNDEFINED_DIMENSION_STYLE = 101
@@ -62,6 +58,8 @@ class AuditError(IntEnum):
     DECODING_ERROR = 111
     CREATED_MISSING_OBJECT = 112
     RESET_MLINE_STYLE = 113
+    INVALID_GROUP_ENTITIES = 114
+    UNDEFINED_BLOCK_NAME = 115
 
     # DXF entity property errors:
     INVALID_ENTITY_HANDLE = 201
@@ -70,6 +68,7 @@ class AuditError(IntEnum):
     INVALID_COLOR_INDEX = 204
     INVALID_LINEWEIGHT = 205
     INVALID_MLINESTYLE_HANDLE = 206
+    INVALID_DIMSTYLE = 207
 
     # DXF entity geometry or content errors:
     INVALID_EXTRUSION_VECTOR = 210
@@ -85,6 +84,8 @@ class AuditError(IntEnum):
     INVALID_SPLINE_KNOT_VALUE_COUNT = 220
     INVALID_SPLINE_WEIGHT_COUNT = 221
     INVALID_DIMENSION_GEOMETRY_LOCATION = 222
+    INVALID_TRANSPARENCY = 223
+    INVALID_CREASE_VALUE_COUNT = 224
 
 
 REQUIRED_ROOT_DICT_ENTRIES = ("ACAD_GROUP", "ACAD_PLOTSTYLENAME")
@@ -95,25 +96,24 @@ class ErrorEntry:
         self,
         code: int,
         message: str = "",
-        dxf_entity: "DXFEntity" = None,
+        dxf_entity: DXFEntity = None,
         data: Any = None,
     ):
         self.code: int = code  # error code AuditError()
-        self.entity: "DXFEntity" = dxf_entity  # source entity of error
+        self.entity: Optional[DXFEntity] = dxf_entity  # source entity of error
         self.message: str = message  # error message
         self.data: Any = data  # additional data as an arbitrary object
 
 
 class Auditor:
-    def __init__(self, doc: Optional["Drawing"]):
-        self.doc: Optional["Drawing"] = doc
-        self._rootdict_handle = doc.rootdict.dxf.handle if doc else "0"
+    def __init__(self, doc: "Drawing"):
+        assert doc is not None
+        self.doc = doc
+        self._rootdict_handle = doc.rootdict.dxf.handle
         self.errors: List[ErrorEntry] = []
         self.fixes: List[ErrorEntry] = []
-        self._trashcan: Optional["EntityDB.Trashcan"] = (
-            doc.entitydb.new_trashcan() if doc else None
-        )
-        self._post_audit_jobs = []
+        self._trashcan = doc.entitydb.new_trashcan()
+        self._post_audit_jobs: List[Callable[[], None]] = []
 
     def reset(self) -> None:
         self.errors = []
@@ -198,7 +198,7 @@ class Auditor:
         self,
         code: int,
         message: str = "",
-        dxf_entity: "DXFEntity" = None,
+        dxf_entity: DXFEntity = None,
         data: Any = None,
     ) -> None:
         self.errors.append(ErrorEntry(code, message, dxf_entity, data))
@@ -207,7 +207,7 @@ class Auditor:
         self,
         code: int,
         message: str = "",
-        dxf_entity: "DXFEntity" = None,
+        dxf_entity: DXFEntity = None,
         data: Any = None,
     ) -> None:
         self.fixes.append(ErrorEntry(code, message, dxf_entity, data))
@@ -222,25 +222,27 @@ class Auditor:
         self.errors = [err for err in self.errors if err.code in codes]
 
     def run(self) -> List[ErrorEntry]:
-        # Check database integrity:
         self.doc.entitydb.audit(self)
         self.check_root_dict()
         self.check_tables()
         self.audit_all_database_entities()
+        self.doc.objects.audit(self)
+        self.doc.blocks.audit(self)
         self.doc.groups.audit(self)
         self.check_block_reference_cycles()
         self.doc.layouts.audit(self)
         self.empty_trashcan()
+        self.doc.objects.purge()
         return self.errors
 
     def empty_trashcan(self):
         if self.has_trashcan:
             self._trashcan.clear()
 
-    def trash(self, entity: "DXFEntity") -> None:
+    def trash(self, entity: DXFEntity) -> None:
         if entity is None or not entity.is_alive:
             return
-        if self.has_trashcan:
+        if self.has_trashcan and entity.dxf.handle is not None:
             self._trashcan.add(entity.dxf.handle)
         else:
             entity.destroy()
@@ -275,7 +277,7 @@ class Auditor:
                 self.entitydb.add(head)
                 self.fixed_error(
                     code=AuditError.INVALID_TABLE_HANDLE,
-                    message=f"Fixed invalid table handle in {table.name}",
+                    message=f"Fixed invalid handle in table {table.name}",
                 )
             # Just to be sure owner handle is valid in every circumstance:
             table.update_owner_handles()
@@ -314,14 +316,14 @@ class Auditor:
             call()
         self._post_audit_jobs = []
 
-    def check_entity_linetype(self, entity: "DXFEntity") -> None:
+    def check_entity_linetype(self, entity: DXFEntity) -> None:
         """Check for usage of undefined line types. AutoCAD does not load
         DXF files with undefined line types.
         """
         assert self.doc is entity.doc, "Entity from different DXF document."
         if not entity.dxf.hasattr("linetype"):
             return
-        linetype = table_key(entity.dxf.linetype)
+        linetype = validator.make_table_key(entity.dxf.linetype)
         # No table entry in linetypes required:
         if linetype in ("bylayer", "byblock"):
             return
@@ -336,7 +338,7 @@ class Auditor:
                 data=linetype,
             )
 
-    def check_text_style(self, entity: "DXFEntity") -> None:
+    def check_text_style(self, entity: DXFEntity) -> None:
         """Check for usage of undefined text styles."""
         assert self.doc is entity.doc, "Entity from different DXF document."
         if not entity.dxf.hasattr("style"):
@@ -364,12 +366,12 @@ class Auditor:
             self.fixed_error(
                 code=AuditError.UNDEFINED_DIMENSION_STYLE,
                 message=f'Replaced undefined dimstyle "{dimstyle}" in '
-                        f'{str(entity)} by "Standard".',
+                f'{str(entity)} by "Standard".',
                 dxf_entity=entity,
                 data=dimstyle,
             )
 
-    def check_for_valid_layer_name(self, entity: "DXFEntity") -> None:
+    def check_for_valid_layer_name(self, entity: DXFEntity) -> None:
         """Check layer names for invalid characters: <>/\":;?*|='"""
         name = entity.dxf.layer
         if not validator.is_valid_layer_name(name):
@@ -405,7 +407,7 @@ class Auditor:
                 dxf_entity=entity,
             )
 
-    def check_owner_exist(self, entity: "DXFEntity") -> None:
+    def check_owner_exist(self, entity: DXFEntity) -> None:
         assert self.doc is entity.doc, "Entity from different DXF document."
         if not entity.dxf.hasattr("owner"):
             return
@@ -434,11 +436,11 @@ class Auditor:
                 self.fixed_error(
                     code=AuditError.INVALID_OWNER_HANDLE,
                     message=f"Deleted {str(entity)} entity with invalid owner "
-                            f"handle #{owner_handle}.",
+                    f"handle #{owner_handle}.",
                 )
                 self.trash(doc.entitydb.get(handle))
 
-    def check_extrusion_vector(self, entity: "DXFEntity") -> None:
+    def check_extrusion_vector(self, entity: DXFEntity) -> None:
         if NULLVEC.isclose(entity.dxf.extrusion):
             entity.dxf.discard("extrusion")
             self.fixed_error(
@@ -447,6 +449,18 @@ class Auditor:
                 dxf_entity=entity,
             )
 
+    def check_transparency(self, entity: DXFEntity) -> None:
+        value = entity.dxf.transparency
+        if value is None:
+            return
+        if not validator.is_transparency(value):
+            entity.dxf.discard("transparency")
+            self.fixed_error(
+                code=AuditError.INVALID_TRANSPARENCY,
+                message=f"Fixed invalid transparency for entity: {str(self)}.",
+                dxf_entity=entity,
+            )
+
     def check_block_reference_cycles(self) -> None:
         cycle_detector = BlockCycleDetector(self.doc)
         for block in self.doc.blocks:
@@ -454,7 +468,7 @@ class Auditor:
                 self.add_error(
                     code=AuditError.INVALID_BLOCK_REFERENCE_CYCLE,
                     message=f"Invalid block reference cycle detected in "
-                            f'block "{block.name}".',
+                    f'block "{block.name}".',
                     dxf_entity=block.block_record,
                 )
 
@@ -492,12 +506,12 @@ class BlockCycleDetector:
             path.pop()
             return False
 
-        path = []
+        path: List[str] = []
         block_name = self.key(block_name)
         return check(block_name)
 
 
-def audit(entity: "DXFEntity", doc: "Drawing") -> Auditor:
+def audit(entity: DXFEntity, doc: "Drawing") -> Auditor:
     """Setup an :class:`Auditor` object, run the audit process for `entity`
     and return result as :class:`Auditor` object.
 
