@@ -1,4 +1,4 @@
-#  Copyright (c) 2021, Manfred Moitzi
+#  Copyright (c) 2021-2022, Manfred Moitzi
 #  License: MIT License
 from typing import Callable, Optional, Dict, TYPE_CHECKING, Type
 import abc
@@ -240,6 +240,18 @@ def load_document(filename: str):
     return doc, auditor
 
 
+HELP_LTYPE = (
+    "select the line type rendering method, default is approximate. "
+    "Approximate uses the closest approximation available to the "
+    "backend, the accurate method renders as accurately as possible "
+    "but this approach is slower."
+)
+HELP_LWSCALE = (
+    "set custom line weight scaling, default is 0 to disable line "
+    "weights at all"
+)
+
+
 @register
 class Draw(Command):
     """Launcher sub-command: draw"""
@@ -290,9 +302,9 @@ class Draw(Command):
         )
         parser.add_argument(
             "--ltype",
-            default="internal",
-            choices=["internal", "ezdxf"],
-            help="select the line type rendering engine, default is internal",
+            default="approximate",
+            choices=["approximate", "accurate"],
+            help=HELP_LTYPE,
         )
 
     @staticmethod
@@ -306,6 +318,7 @@ class Draw(Command):
 
         from ezdxf.addons.drawing import RenderContext, Frontend
         from ezdxf.addons.drawing.matplotlib import MatplotlibBackend
+        from ezdxf.addons.drawing.config import Configuration, LinePolicy
 
         # Print all supported export formats:
         if args.formats:
@@ -337,12 +350,19 @@ class Draw(Command):
         fig = plt.figure()
         ax = fig.add_axes([0, 0, 1, 1])
         ctx = RenderContext(doc)
-        out = MatplotlibBackend(ax, params={"linetype_renderer": args.ltype})
+        out = MatplotlibBackend(ax)
 
         if args.all_layers_visible:
             for layer_properties in ctx.layers.values():
                 layer_properties.is_visible = True
 
+        config = Configuration.defaults()
+        config = config.with_changes(
+            line_policy=LinePolicy.ACCURATE
+            if args.ltype == "accurate"
+            else config.line_policy
+        )
+
         if args.all_entities_visible:
 
             class AllVisibleFrontend(Frontend):
@@ -351,9 +371,9 @@ class Draw(Command):
                 ) -> None:
                     properties.is_visible = True
 
-            frontend = AllVisibleFrontend(ctx, out)
+            frontend = AllVisibleFrontend(ctx, out, config=config)
         else:
-            frontend = Frontend(ctx, out)
+            frontend = Frontend(ctx, out, config=config)
         frontend.draw_layout(layout, finalize=True)
 
         if args.out is not None:
@@ -388,47 +408,41 @@ class View(Command):
         )
         parser.add_argument(
             "--ltype",
-            default="internal",
-            choices=["internal", "ezdxf"],
-            help="select the line type rendering engine, default is internal",
+            default="approximate",
+            choices=["approximate", "accurate"],
+            help=HELP_LTYPE,
         )
         # disable lineweight at all by default:
         parser.add_argument(
             "--lwscale",
             type=float,
             default=0,
-            help="set custom line weight scaling, default is 0 to disable "
-            "line weights at all",
-        )
-        parser.add_argument(
-            "--scale",
-            type=float,
-            default=1.0,
-            help="set set overall scaling factor > 1e-9 to scale the content",
+            help=HELP_LWSCALE,
         )
 
     @staticmethod
     def run(args):
         # Import on demand for a quicker startup:
         try:
-            from PyQt5 import QtWidgets
-        except ImportError:
-            print("PyQt5 package not found.")
+            from ezdxf.addons.xqt import QtWidgets
+        except ImportError as e:
+            print(str(e))
             sys.exit(1)
         from ezdxf.addons.drawing.qtviewer import CadViewer
+        from ezdxf.addons.drawing.config import Configuration, LinePolicy
+
+        config = Configuration.defaults()
+        config = config.with_changes(
+            line_policy=LinePolicy.ACCURATE
+            if args.ltype == "accurate"
+            else config.line_policy,
+            lineweight_scaling=args.lwscale,
+        )
 
-        if args.scale < 1e-9:
-            print("Scaling factor too small or negative!")
-            sys.exit(2)
         signal.signal(signal.SIGINT, signal.SIG_DFL)  # handle Ctrl+C properly
         app = QtWidgets.QApplication(sys.argv)
         set_app_icon(app)
-        viewer = CadViewer(
-            params={
-                "linetype_renderer": args.ltype,
-                "lineweight_scaling": args.lwscale,
-            }
-        )
+        viewer = CadViewer(config=config)
         filename = args.file
         if filename:
             doc, auditor = load_document(filename)
@@ -436,9 +450,8 @@ class View(Command):
                 doc,
                 auditor,
                 layout=args.layout,
-                overall_scaling_factor=args.scale,
             )
-        sys.exit(app.exec_())
+        sys.exit(app.exec())
 
 
 @register
@@ -472,20 +485,23 @@ class Browse(Command):
     @staticmethod
     def run(args):
         try:
-            from PyQt5.QtWidgets import QApplication
-        except ImportError:
-            print("PyQt5 package not found.")
+            from ezdxf.addons.xqt import QtWidgets
+        except ImportError as e:
+            print(str(e))
             sys.exit(1)
         from ezdxf.addons import browser
 
         signal.signal(signal.SIGINT, signal.SIG_DFL)  # handle Ctrl+C properly
-        app = QApplication(sys.argv)
+        app = QtWidgets.QApplication(sys.argv)
         set_app_icon(app)
         main_window = browser.DXFStructureBrowser(
-            args.file, line=args.line, handle=args.handle
+            args.file,
+            line=args.line,
+            handle=args.handle,
+            resource_path=resources_path(),
         )
         main_window.show()
-        sys.exit(app.exec_())
+        sys.exit(app.exec())
 
 
 @register
@@ -583,8 +599,58 @@ class Config(Command):
             options.print()
 
 
+@register
+class Info(Command):
+    """Launcher sub-command: info"""
+
+    NAME = "info"
+
+    @staticmethod
+    def add_parser(subparsers):
+        parser = subparsers.add_parser(
+            Info.NAME, help="show information and optional stats of DXF files as "
+                            "loaded by ezdxf, this may not represent the original "
+                            "content of the file, use the browse command to "
+                            "see the original content"
+        )
+        parser.add_argument(
+            "file",
+            metavar="FILE",
+            nargs="+",
+            help='DXF file to process, wildcards "*" and "?" are supported',
+        )
+        parser.add_argument(
+            "-v",
+            "--verbose",
+            action="store_true",
+            required=False,
+            help="give more output",
+        )
+        parser.add_argument(
+            "-s",
+            "--stats",
+            action="store_true",
+            required=False,
+            help="show content stats",
+        )
+
+    @staticmethod
+    def run(args):
+        from ezdxf.document import info
+
+        for pattern in args.file:
+            for filename in glob.glob(pattern):
+                doc, auditor = load_document(filename)
+                print(
+                    "\n".join(
+                        info(doc, verbose=args.verbose, content=args.stats)
+                    )
+                )
+                print()
+
+
 def set_app_icon(app):
-    from PyQt5 import QtGui, QtCore
+    from ezdxf.addons.xqt import QtGui, QtCore
 
     app_icon = QtGui.QIcon()
     p = resources_path()
